@@ -25,7 +25,6 @@ spca <- function(obj, xy=NULL, cn=NULL, matWeight=NULL,
     if(!any(inherits(obj,c("genind","genpop")))) stop("obj must be a genind or genpop object.")
     invisible(validObject(obj))
     ## checkType(obj)
-    if(!require(spdep, quietly=TRUE)) stop("spdep library is required.")
 
 
     ## handle xy coordinates
@@ -88,7 +87,7 @@ spca <- function(obj, xy=NULL, cn=NULL, matWeight=NULL,
     }
 
     ## handle NAs, centring and scaling
-    X <- scaleGen(obj, center=TRUE, scale=scale, missing="mean", truenames=truenames)
+    X <- scaleGen(obj, center=TRUE, scale=scale, NA.method="mean")
 
     ## perform analyses
     pcaX <- dudi.pca(X, center=FALSE, scale=FALSE, scannf=FALSE)
@@ -189,7 +188,6 @@ print.spca <- function(x, ...){
 ########################
 summary.spca <- function (object, ..., printres=TRUE) {
   if (!inherits(object, "spca"))stop("to be used with 'spca' object")
-  if(!require(spdep,quietly=TRUE)) stop("the library spdep is required; please install this package")
 
   #util <- function(n) { ## no longer used
   #  x <- "1"
@@ -223,15 +221,10 @@ summary.spca <- function (object, ..., printres=TRUE) {
   }
 
   if(is.genind(obj)) { X <- obj@tab }
-  if(is.genpop(obj)) { X <- makefreq(obj, quiet=TRUE)$tab }
+  if(is.genpop(obj)) { X <- makefreq(obj, quiet=TRUE) }
 
   X <- apply(X,2,f1)
 
-  if(appel$truenames){
-    rownames(X) <- rownames(truenames(obj))
-    colnames(X) <- colnames(truenames(obj))
-  }
-
   nfposi <- object$nfposi
   nfnega <- object$nfnega
 
@@ -313,7 +306,6 @@ summary.spca <- function (object, ..., printres=TRUE) {
 plot.spca <- function (x, axis = 1, useLag=FALSE, ...){
     if (!inherits(x, "spca")) stop("Use only with 'spca' objects.")
 
-    if(!require(spdep)) stop("spdep package is required.")
     if(axis>ncol(x$li)) stop("wrong axis required.")
 
     opar <- par(no.readonly = TRUE)
